#include <guile_fixups.h>
#if SCM_MAJOR_VERSION >= 2
#include <assert.h>

static SCM
eval_str_wrapper (void *data)
{
  char *scheme_code = (char *) data;
  return gh_eval_str (scheme_code);
}

static SCM
eval_file_wrapper (void *data)
{
/*   gh_eval_t real_eval_proc = (gh_eval_t) (* ((gh_eval_t *) data)); */

  char *scheme_code = (char *) data;
  return gh_eval_file (scheme_code);
}

SCM
gh_eval_str_with_catch (const char *scheme_code, scm_t_catch_handler handler)
{
	/* FIXME: not there yet */
	return gh_catch (SCM_BOOL_T, (scm_t_catch_body) eval_str_wrapper, (void *) scheme_code,
			(scm_t_catch_handler) handler, (void *) scheme_code);
}

SCM
gh_eval_file_with_catch (const char *scheme_code, scm_t_catch_handler handler)
{
  /* FIXME: not there yet */
  return gh_catch (SCM_BOOL_T, (scm_t_catch_body) eval_file_wrapper,
		   (void *) scheme_code, (scm_t_catch_handler) handler,
		   (void *) scheme_code);
}

/* This function takes care of all real GH initialization.  Since it's
   called by scm_boot_guile, it can safely work with heap objects, or
   call functions that do so.  */
static void 
gh_launch_pad (void *closure, int argc, char **argv)
{
  main_prog_t c_main_prog = (main_prog_t) closure;

  c_main_prog (argc, argv);
  exit (0);
}

/* starts up the Scheme interpreter, and stays in it.  c_main_prog()
   is the address of the user's main program, since gh_enter() never
   returns. */
void 
gh_enter (int argc, char *argv[], main_prog_t c_main_prog)
{
  scm_boot_guile (argc, argv, gh_launch_pad, (void *) c_main_prog);
  /* never returns */
}

static void *
scm2whatever (SCM obj, void *m, size_t size)
{
  scm_t_array_handle handle;
  size_t len;
  ssize_t inc;
  const void *elts;

  elts = scm_uniform_vector_elements (obj, &handle, &len, &inc);

  if (inc != 1)
    scm_misc_error (NULL, "only contiguous vectors can be converted: ~a",
		    scm_list_1 (obj));

  if (m == 0)
    m = malloc (len * sizeof (size));
  if (m != NULL)
    memcpy (m, elts, len * size);

  scm_array_handle_release (&handle);

  return m;
}

#define SCM2WHATEVER(obj,pred,utype,mtype)                   \
  if (scm_is_true (pred (obj)))                              \
    {                                                        \
      assert (sizeof (utype) == sizeof (mtype));             \
      return (mtype *)scm2whatever (obj, m, sizeof (utype)); \
    }

/* Convert a vector, weak vector or uniform vector into an array of
   doubles.  If result array in arg 2 is NULL, malloc a new one.  If
   out of memory, return NULL.  */
double *
gh_scm2doubles (SCM obj, double *m)
{
  long i, n;
  SCM val;
  if (SCM_IMP (obj))
    scm_wrong_type_arg (0, 0, obj);

  /* XXX - f32vectors are rejected now.
   */
  SCM2WHATEVER (obj, scm_f64vector_p, double, double)

  switch (SCM_TYP7 (obj))
    {
    case scm_tc7_vector:
    case scm_tc7_wvect:
      n = SCM_SIMPLE_VECTOR_LENGTH (obj);
      for (i = 0; i < n; ++i)
	{
	  val = SCM_SIMPLE_VECTOR_REF (obj, i);
	  if (!SCM_I_INUMP (val)
	      && !(SCM_BIGP (val) || SCM_REALP (val)))
	    scm_wrong_type_arg (0, 0, val);
	}
      if (m == 0)
	m = (double *) malloc (n * sizeof (double));
      if (m == NULL)
	return NULL;
      for (i = 0; i < n; ++i)
	{
	  val = SCM_SIMPLE_VECTOR_REF (obj, i);
	  if (SCM_I_INUMP (val))
	    m[i] = SCM_I_INUM (val);
	  else if (SCM_BIGP (val))
	    m[i] = scm_to_long (val);
	  else
	    m[i] = SCM_REAL_VALUE (val);
	}
      break;

    default:
      scm_wrong_type_arg (0, 0, obj);
    }
  return m;
}

#endif /* SCM_MAJOR_VERSION */
